# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'tempfile'
require 'fileutils'
require 'puppet_x/enterprisemodules/postgres/sql'
require 'puppet_x/enterprisemodules/postgres/settings'

module PuppetX
  module EnterpriseModules
    module Postgres
      # Docs
      # rubocop: disable Metrics/ModuleLength
      module Access
        include EasyType::Helpers
        include Settings

        def module_name
          'pg_config'
        end

        def self.included(parent)
          parent.extend(Access)
        end

        def for_version(hash)
          hash.collect do |version, query|
            [lambda { |db| pg_version?(db, version) }, query]
          end
        end

        #
        # Check if the Postgres version of the specified db is equal to the
        # specified version.
        #
        def pg_version?(db, version)
          return true if version.to_s == 'default'

          if version.instance_of?(String) || version.instance_of?(Symbol)
            Postgres_major_version(db) == version.to_s.to_i
          elsif version.instance_of?(Regexp)
            database_version(db) =~ version
          else
            fail 'Invalid type specified for sql version query'
          end
        end

        ##
        #
        # Use this function to execute Postgres statements on all running clusterdatabases.
        #
        # @param command [String] this is the commands to be given
        #
        #
        def sql_on_cluster_databases(command, parameters = {})
          sql_on_instances(registered_instances, command, parameters.merge(:db => 'postgres'))
        end

        ##
        #
        # Get a list of all databases on an instance
        #
        def databases_on_instance(instance)
          all_databases = sql('select datname as database from pg_database', :instance => instance, :db => 'postgres').collect { |e| e['database'] }
          # Template 0 cannot b connected to.
          all_databases - ['template0']
        end

        ##
        #
        # Use this function to execute Postgres statements on all running databases.
        #
        # @param command [String] this is the commands to be given
        #
        #
        def sql_on_all_databases(command, parameters = {})
          results = []
          registered_instances.each do |instance|
            databases_on_instance(instance).each do |db|
              results += sql(command, { :instance => instance, :db => db }.merge(parameters))
            end
          end
          results
        end

        #
        # Run the sql commmand on all specified instances. If the sql is a String,
        # The string will be used as the sql statement.
        #
        # The command, can also be an array. In that case, every pair contains
        # a landa followed by an sql sattement e.g
        #
        #  [
        #      lamda { a check},         'select * from tab',
        #      lamda { an other  check}, 'select * from users',
        #  ]
        #
        # The check will be executed on every db. You can use this for example
        # to use different statements on different versions of Postgres
        #
        # @param instances [Array] Array of instances to run the command on
        # @param command [String] Sql statement to run on all instances
        #
        # or
        # @param command [Array] Array of lambda's and sql statements.
        #
        # @return [Array] The returned results
        #
        def sql_on_instances(instances, command, parameters = {})
          results = []
          instances.each do |instance|
            results += sql(command, { :instance => instance }.merge(parameters))
          end
          results
        end

        # @private
        #
        # Returns the first valid sql statement in the versioned command
        #
        def select_statement(versioned_command, db)
          versioned_command.each do |version_proc, statement|
            return statement if version_proc.call(db)
          end
          fail 'no valid version found'
        end

        # @private
        def versioned_statement?(command)
          command.is_a?(::Array)
        end

        ##
        #
        # Use this function to execute Postgres statements
        #
        # @param command [String] this is the commands to be given
        #
        #
        # rubocop: disable Metrics/MethodLength
        def sql(command, options = {})
          options[:timeout] ||= resource[:timeout] if timeout_specified
          parse = options.fetch(:parse, true)
          @sql = Sql.new(options)
          instance = @sql.instance
          db = @sql.db
          statement = if versioned_statement?(command)
                        select_statement(command, instance)
                      else
                        command
                      end
          if statement[0] == '@'
            # rubocop: disable Style/SlicingWithRange
            file_name = statement[1..-1]
            # rubocop: enable Style/SlicingWithRange
            fail "Input file '#{file_name}' not found." unless File.exist?(file_name)

            statement = "\\i #{file_name}" if statement[0] == '@'
          end
          Puppet.debug "executing #{statement} on instace #{instance}"
          results = @sql.execute(statement).gsub("\n\n", "\n")
          if parse
            add_instance_and_db_to(convert_csv_data_to_hash(results, [], :col_sep => ',', :converters => lambda { |f| f ? f.strip : nil }), instance, db)
          else
            results
          end
        end
        # rubocop: enable Metrics/MethodLength

        def add_instance_and_db_to(elements, instance, db)
          elements.collect do |e|
            e['instance'] = instance
            e['db'] = db
            e
          end
        end

        # This is a little hack to get a specified timeout value
        def timeout_specified
          resource.timeout
        rescue NameError
          nil
        end

        def pg_autorequire(type, property)
          autorequire(type) do
            if property.is_a?(Array)
              property.collect { |p| resource_list_for(p) }.flatten
            else
              resource_list_for(property)
            end
          end
        end

        def resource_list_for(property)
          value = property.is_a?(Proc) ? instance_exec(&property) : send(property)
          current_instance = self[:instance]
          if value.nil? || value.empty?
            nil
          elsif value.is_a?(::Array)
            if current_instance
              value.collect { |element| "#{element}@#{current_instance}" }.flatten
            else
              value
            end
          else
            current_instance ? "#{value}@#{current_instance}" : value
          end
        end

        def instance
          if resource[:instances].any?
            resource[:instances].select { |_inst, host| host.include? Facter.value('hostname') }.to_h.keys[0]
          else
            resource.name
          end
        end
      end
      # rubocop: enable Metrics/ModuleLength
    end
  end
end
