# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'tmpdir'
require 'puppet_x/enterprisemodules/postgres/settings'
module PuppetX
  module EnterpriseModules
    module Postgres
      # docs
      class Command
        include Settings

        DEFAULT_TIMEOUT = 300 # 5 minutes

        COMMAND_VALID_OPTIONS = [
          :instance,
          :db,
          :os_user,
          :db_user,
          :password,
          :timeout,
          :username
        ].freeze

        attr_reader(*COMMAND_VALID_OPTIONS)

        def initialize(command, options, valid_options = COMMAND_VALID_OPTIONS)
          @valid_options = valid_options
          check_options(options)
          @command  = command
          @password = options[:password] # may be empty
          @timeout  = options.fetch(:timeout) { DEFAULT_TIMEOUT }
          @instance = options.fetch(:instance) { raise ArgumentError, 'you need to specify a instance for postgres access' }
          @os_user  = options.fetch(:os_user) { configuration_value_for(@instance, 'os_user') }
          @username = options.fetch(:username, nil)
          @db       = options.fetch(:db) { raise ArgumentError, 'you need to specify a db for postgres access' }
        end

        def command_string(arguments = '')
          "su - #{@os_user} -c \"cd #{working_dir}; #{@command} #{arguments}\""
        end

        def execute(arguments)
          options = { :failonfail => true, :combine => true }
          value = ''
          within_time(@timeout) do
            Puppet.debug "command.rb: Executing #{@command} command: #{arguments} on #{@instance} as #{os_user}, connected as #{username}"
            value = Puppet::Util::Execution.execute(command_string(arguments), options)
          end
          value
        end

        protected

        def working_dir
          pwd = Dir.pwd
          unless writable_by_user(@os_user, pwd)
            pwd = Dir.tmpdir
            Puppet.debug "command.rb: Running Postgres command from a pwd that is not writable for os user #{os_user} therefore resetting pwd to '#{pwd}'"
          end
          pwd
        end

        private

        def writable_by_user(os_user, pwd)
          # TODO: what mechanisme do we need for Windows
          if Puppet::Util::Platform.windows?
            true
          else
            system "su - #{os_user} -c \"cd #{pwd};\" > /dev/null 2>&1"
          end
        end

        def within_time(timeout, &block)
          Puppet.debug "command.rb: Using timeout #{timeout}"
          if timeout.zero?
            yield
          else
            Timeout.timeout(timeout, &block)
          end
        end

        def check_options(options)
          options.each_key { |key| raise ArgumentError, "option #{key} invalid for #{@command}. Only #{@valid_options.join(', ')} are supported" unless @valid_options.include?(key) }
        end
      end
    end
  end
end
