# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/postgres/command'
require 'puppet_x/enterprisemodules/postgres/settings'
require 'easy_type'
require 'easy_type/encryption'
# rubocop: disable Style/ClassVars

module PuppetX
  module EnterpriseModules
    module Postgres
      # Docs
      class PsqlCommand < PuppetX::EnterpriseModules::Postgres::Command
        include EasyType::Template
        include Settings
        include EasyType::Encryption

        @@created_files ||= []

        PSQL_VALID_OPTIONS = [
          :instance,
          :db,
          :os_user,
          :db_user,
          :password,
          :timeout,
          :username,
          :failonsqlfail, # Don't fail if the sql fails
          :catch_errors,
          :parse, # When parse is false, the output is not mment for parsing
          :catch_extra_errors
        ].freeze

        attr_reader(*PSQL_VALID_OPTIONS)

        def initialize(options = {})
          @failonsqlfail = options.fetch(:failonsqlfail, true)
          @catch_errors       = options.fetch(:catch_errors) { /ERROR/ }
          @catch_extra_errors = options[:catch_extra_errors]
          @catch_errors       = Regexp.union(@catch_errors, @catch_extra_errors) if @catch_extra_errors
          @parse              = options.fetch(:parse, true)
          @db                 = options[:db]
          super('psql  --pset footer ', options, PSQL_VALID_OPTIONS)
          #
          # Set all values based on settings
          #
          @os_user = configuration_value_for(@instance, 'os_user', 'postgres')
          @db_user = configuration_value_for(@instance, 'db_user', 'postgres')
          @port    = configuration_value_for(@instance, 'port')
        end

        def command_string(arguments = '')
          "su - #{@os_user} -c \"cd #{working_dir}; #{@command} -d #{@db} -U #{@db_user} -p #{@port} #{arguments}\""
        end

        # rubocop: disable Metrics/MethodLength
        def execute(arguments)
          options = { :failonfail => true, :combine => true }
          value = ''
          command = command_string(arguments)
          within_time(@timeout) do
            Puppet.debug "PSQL_command.rb: Executing #{@command} command: #{arguments} as user #{@os_user}"
            value = Puppet::Util::Execution.execute(command, options)
            Puppet.debug "PSQL_command.rb: Output:\n #{value}"
          end
          value.encode!('UTF-8', :invalid => :replace, :undef => :replace)
          output_file = Tempfile.new(['sql', '.log'])
          ObjectSpace.undefine_finalizer(output_file) # Don't delete the file
          output_file.write(value)
          output_file.close
          FileUtils.chown(@os_user, nil, output_file.path)
          FileUtils.chmod(0o600, output_file.path)
          FileUtils.chown(@os_user, nil, output_file.path)
          FileUtils.chmod(0o600, output_file.path)
          path = output_file.path
          @@created_files << path
          Puppet.debug "PSQL_command.rb: SQL output saved to #{output_file.path}"
          scan_for_errors(value) if @failonsqlfail
          value
        end
        # rubocop: enable Metrics/MethodLength

        def execute_sql_command(command, output_file)
          #
          # Do the stuff
          #
          Puppet.debug "PSQL_command.rb: Executing sql statement :\n #{command}"
          script = command_file(template('pg_config/pg_config/execute.sql.erb', binding))
          if username
            Puppet.debug "PSQL_command.rb: Connecting to postgres instance #{@instance} as user #{username}"
          else
            Puppet.debug "PSQL_command.rb: Connecting to postgres instance #{@instance} as user #{@db_username}"
          end
          execute("-f #{script}")
          content = File.read(output_file)
          content.encode!('UTF-8', :invalid => :replace, :undef => :replace)
          content
        end

        private

        def scan_for_errors(string)
          fail "error in sql statement processing\n #{string}" if string.scan(@catch_errors).any?
        end

        def command_file(content, type = '.sql')
          command_file = Tempfile.new(['command', type])
          ObjectSpace.undefine_finalizer(command_file) # Don't delete the file
          command_file.write(content)
          command_file.close
          FileUtils.chown(@os_user, nil, command_file.path)
          FileUtils.chmod(0o600, command_file.path)
          FileUtils.chown(@os_user, nil, command_file.path)
          FileUtils.chmod(0o600, command_file.path)
          path = command_file.path
          @@created_files << path
          path
        end

        at_exit do
          #
          # If environment variable pg_config_KEEP is set to yes,
          # we won't remove the files else we will.
          #
          @@created_files.each { |cmd_file| FileUtils.rm(cmd_file) } unless ENV['PG_CONFIG_KEEP'].to_s.casecmp('yes').zero?
        end
      end
    end
  end
end
# rubocop: enable Style/ClassVars
